using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BearWare;

namespace TeamTalk.NET
{
    public partial class UserAccountsDlg : Form
    {
        TeamTalk4 ttclient;
        int create_cmdid;
        int list_cmdid;
        int del_cmdid;

        public UserAccountsDlg(TeamTalk4 tt)
        {
            ttclient = tt;
            create_cmdid = 0;
            del_cmdid = 0;

            InitializeComponent();
            this.CenterToScreen();

            listView1.Columns.Add("Username");
            listView1.Columns.Add("Password");
            listView1.Columns.Add("User Type");
            listView1.Columns.Add("UserData");
            listView1.Columns.Add("Note");

            ttclient.OnCmdProcessing += new TeamTalk4.CommandProcessing(ttclient_OnCmdProcessing);

            list_cmdid = ttclient.DoListUserAccounts(0, 100000);
        }

        void ttclient_OnCmdProcessing(int nCmdID, bool bComplete)
        {
            if (create_cmdid == nCmdID && bComplete)
            {
                createButton.Enabled = true;
                usernameTextBox.ReadOnly = false;
                passwordTextBox.ReadOnly = false;
                defuserRadioButton.Enabled = true;
                adminRadioButton.Enabled = true;
                noteTextBox.Enabled = true;
                userdataTextBox.Enabled = true;

                UserAccount account;
                account.szUsername = usernameTextBox.Text;
                account.szPassword = passwordTextBox.Text;
                account.szNote = noteTextBox.Text;
                account.nUserData = int.Parse(userdataTextBox.Text);

                account.uUserType = adminRadioButton.Checked ? UserType.USERTYPE_ADMIN : UserType.USERTYPE_DEFAULT;
                AddAccount(account);
                usernameTextBox.Text = "";
                passwordTextBox.Text = "";
                noteTextBox.Text = "";
                userdataTextBox.Text = "0";
                defuserRadioButton.Checked = true;
                create_cmdid = 0;
            }
            else if (list_cmdid == nCmdID && bComplete)
            {
                UserAccount[] accounts;
                if (ttclient.GetUserAccounts(out accounts))
                {
                    foreach (UserAccount account in accounts)
                        AddAccount(account);
                }
                list_cmdid = 0;
            }
            else if (del_cmdid == nCmdID && bComplete)
            {
                listView1.Enabled = true;
                if (listView1.SelectedItems.Count > 0)
                {
                    listView1.Items.Remove(listView1.SelectedItems[0]);
                }
            }
        }

        private void createButton_Click(object sender, EventArgs e)
        {
            if(usernameTextBox.Text.Length == 0)
                return;

            UserAccount account;
            account.szUsername = usernameTextBox.Text;
            account.szPassword = passwordTextBox.Text;
            if(adminRadioButton.Checked)
                account.uUserType = UserType.USERTYPE_ADMIN;
            else
                account.uUserType = UserType.USERTYPE_DEFAULT;
            account.szNote = noteTextBox.Text;
            account.nUserData = int.Parse(userdataTextBox.Text);

            create_cmdid = ttclient.DoNewUserAccount(account);
            if (create_cmdid > 0)
            {
                createButton.Enabled = false;
                usernameTextBox.ReadOnly = true;
                passwordTextBox.ReadOnly = true;
                defuserRadioButton.Enabled = false;
                adminRadioButton.Enabled = false;
                userdataTextBox.Enabled = false;
                noteTextBox.Enabled = false;
            }
        }

        private void deleteAccountToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                del_cmdid = ttclient.DoDeleteUserAccount(listView1.SelectedItems[0].Text);
                listView1.Enabled = false;
            }
        }

        void AddAccount(UserAccount account)
        {
            ListViewItem item = new ListViewItem();
            item.Text = account.szUsername;
            item.SubItems.Add(account.szPassword);
            item.SubItems.Add((account.uUserType & UserType.USERTYPE_ADMIN) == UserType.USERTYPE_ADMIN ? "Admin" : "Default");
            item.SubItems.Add(account.nUserData.ToString());
            item.SubItems.Add(account.szNote);
            listView1.Items.Add(item);
        }
    }
}