using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BearWare;

namespace TeamTalk.NET
{
    public enum ChannelDlgType
    {
        JOIN_CHANNEL,
        CREATE_CHANNEL,
        UPDATE_CHANNEL,
        VIEW_CHANNEL
    }

    enum CodecTabs
    {
        SPEEX_CBR_TAB = 0,
        SPEEX_VBR_TAB = 1,
        CELT_CBR_TAB = 2,
        CELT_VBR_TAB = 3
    }

    public partial class ChannelDlg : Form
    {
        TeamTalk4 ttclient;
        ChannelDlgType dlgtype;
        Channel chan;

        public ChannelDlg(TeamTalk4 tt, ChannelDlgType dlgtype, int channelid, int parentid)
        {
            ttclient = tt;
            this.dlgtype = dlgtype;

            InitializeComponent();

            if (channelid > 0 && ttclient.GetChannel(channelid, out chan))
            {
                string path = "";
                ttclient.GetChannelPath(channelid, out path);
                chanpathTextBox.Text = path;
                channameTextBox.Text = chan.szName;
                topicTextBox.Text = chan.szTopic;
                passwdTextBox.Text = chan.szPassword;
                maxusersNumericUpDown.Value = chan.nMaxUsers;
                diskquotaNumericUpDown.Value = chan.nDiskQuota / 1024;
                staticCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_STATIC) == ChannelType.CHANNEL_STATIC;
                echoCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_ECHO) == ChannelType.CHANNEL_ECHO;
                singletxCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_SOLO_TRANSMIT) == ChannelType.CHANNEL_SOLO_TRANSMIT;
                classroomCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_CLASSROOM) == ChannelType.CHANNEL_CLASSROOM;
                echovideoCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_ECHO_VIDEO) == ChannelType.CHANNEL_ECHO_VIDEO;
                echodesktopCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_ECHO_DESKTOP) == ChannelType.CHANNEL_ECHO_DESKTOP;
                oprecvonlyCheckBox.Checked = (chan.uChannelType & ChannelType.CHANNEL_OPERATOR_RECVONLY) == ChannelType.CHANNEL_OPERATOR_RECVONLY;

                switch (chan.codec.nCodec)
                {
                    case Codec.SPEEX_CODEC:
                        tabControl1.SelectedIndex = (int)CodecTabs.SPEEX_CBR_TAB;
                        speexnbRadioButton.Checked = chan.codec.speex.nBandmode == 0;
                        speexwbRadioButton.Checked = chan.codec.speex.nBandmode == 1;
                        speexuwbRadioButton.Checked = chan.codec.speex.nBandmode == 2;
                        speexqualityTrackBar.Value = chan.codec.speex.nQuality;
                        speexTxInterval.Value = chan.codec.speex.nMSecPerPacket;
                        break;
                    case Codec.SPEEX_VBR_CODEC :
                        tabControl1.SelectedIndex = (int)CodecTabs.SPEEX_VBR_TAB;
                        speexvbrnbRadioButton.Checked = chan.codec.speex_vbr.nBandmode == 0;
                        speexvbrwbRadioButton.Checked = chan.codec.speex_vbr.nBandmode == 1;
                        speexvbruwbRadioButton.Checked = chan.codec.speex_vbr.nBandmode == 2;
                        speexvbrqualityTrackBar.Value = chan.codec.speex_vbr.nQualityVBR;
                        speexvbrmaxbrNumericUpDown.Value = chan.codec.speex_vbr.nMaxBitRate;
                        speexvbrTxInterval.Value = chan.codec.speex_vbr.nMSecPerPacket;
                        speexvbrdtxCheckBox.Checked = chan.codec.speex_vbr.bDTX;
                        break;
                    case Codec.CELT_CODEC:
                        tabControl1.SelectedIndex = (int)CodecTabs.CELT_CBR_TAB;
                        celtkhz32RadioButton.Checked = chan.codec.celt.nSampleRate == 32000;
                        celtkhz441RadioButton.Checked = chan.codec.celt.nSampleRate == 44100;
                        celtkhz64RadioButton.Checked = chan.codec.celt.nSampleRate == 64000;
                        celtbitrateNumericUpDown.Value = chan.codec.celt.nBitRate / 1024;
                        celtchannelsComboBox.SelectedIndex = chan.codec.celt.nChannels - 1;
                        celtTxInterval.Value = chan.codec.celt.nMSecPerPacket;
                        break;
                    case Codec.CELT_VBR_CODEC:
                        tabControl1.SelectedIndex = (int)CodecTabs.CELT_VBR_TAB;
                        celtvbrkhz32RadioButton.Checked = chan.codec.celt_vbr.nSampleRate == 32000;
                        celtvbrkhz441RadioButton.Checked = chan.codec.celt_vbr.nSampleRate == 44100;
                        celtvbrkhz64RadioButton.Checked = chan.codec.celt_vbr.nSampleRate == 64000;
                        celtvbrbitrateNumericUpDown.Value = chan.codec.celt_vbr.nBitRate / 1024;
                        celtvbrchannelsComboBox.SelectedIndex = chan.codec.celt_vbr.nChannels - 1;
                        celtvbrTxIntervalNumericUpDown.Value = chan.codec.celt_vbr.nMSecPerPacket;
                        break;
                }

                agcCheckBox.Checked = chan.audiocfg.bEnableAGC;
                gainlevelTrackBar.Value = chan.audiocfg.nGainLevel;
                denoiseCheckBox.Checked = chan.audiocfg.bEnableDenoise;
            }
            else
            {
                string path = "";
                ttclient.GetChannelPath(parentid, out path);
                chanpathTextBox.Text = path;
                chan.nParentID = parentid;
            }

            UserType usertype = ttclient.GetMyUserType();
            switch (dlgtype)
            {
                case ChannelDlgType.JOIN_CHANNEL :
                    //when joining a channel 'disk quota' and 'max users' are set to server's default values
                    diskquotaNumericUpDown.ReadOnly = true;
                    maxusersNumericUpDown.ReadOnly = true;
                    goto case ChannelDlgType.CREATE_CHANNEL;
                case ChannelDlgType.CREATE_CHANNEL :
                    speexwbRadioButton.Checked = true;
                    speexqualityTrackBar.Value = 4;
                    celtchannelsComboBox.SelectedIndex = 0;
                    celtvbrchannelsComboBox.SelectedIndex = 0;
                    //only admins can change static channel property
                    staticCheckBox.Enabled = (usertype & UserType.USERTYPE_ADMIN) == UserType.USERTYPE_ADMIN;
                    diskquotaNumericUpDown.ReadOnly = (usertype & UserType.USERTYPE_ADMIN) != UserType.USERTYPE_ADMIN;
                    agcCheckBox.Checked = true;
                    gainlevelTrackBar.Value = 8000;
                    denoiseCheckBox.Checked = true;
                    break;
                case ChannelDlgType.UPDATE_CHANNEL :
                    //channel name cannot be updated
                    //also codecs cannot be updated if there's users in the channel
                    channameTextBox.ReadOnly = true;
                    break;
                case ChannelDlgType.VIEW_CHANNEL :
                    channameTextBox.ReadOnly = true;
                    topicTextBox.ReadOnly = true;
                    passwdTextBox.ReadOnly = true;
                    oppasswdTextBox.ReadOnly = true;
                    maxusersNumericUpDown.ReadOnly = true;
                    diskquotaNumericUpDown.ReadOnly = true;
                    staticCheckBox.Enabled = false;
                    echoCheckBox.Enabled = false;
                    singletxCheckBox.Enabled = false;
                    classroomCheckBox.Enabled = false;
                    echovideoCheckBox.Enabled = false;
                    echodesktopCheckBox.Enabled = false;
                    oprecvonlyCheckBox.Enabled = false;
                    agcCheckBox.Enabled = false;
                    gainlevelTrackBar.Enabled = false;
                    denoiseCheckBox.Enabled = false;
                    tabControl1.Enabled = false;
                    break;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            chan.szName = channameTextBox.Text;
            chan.szTopic = topicTextBox.Text;
            chan.szPassword = passwdTextBox.Text;
            chan.szOpPassword = oppasswdTextBox.Text;
            chan.nMaxUsers = (int)maxusersNumericUpDown.Value;
            chan.nDiskQuota = (long)diskquotaNumericUpDown.Value * 1024;

            switch(tabControl1.SelectedIndex)
            {
                case (int)CodecTabs.SPEEX_CBR_TAB:

                    chan.codec.nCodec = Codec.SPEEX_CODEC;
                    if (speexnbRadioButton.Checked)
                        chan.codec.speex.nBandmode = 0;
                    else if (speexwbRadioButton.Checked)
                        chan.codec.speex.nBandmode = 1;
                    else if (speexuwbRadioButton.Checked)
                        chan.codec.speex.nBandmode = 2;
                    chan.codec.speex.nQuality = speexqualityTrackBar.Value;
                    chan.codec.speex.nMSecPerPacket = (int)speexTxInterval.Value;
                    chan.codec.speex.bUseJitterBuffer = false;
                    chan.codec.speex.bStereoPlayback = false;
                    break;

                case (int)CodecTabs.SPEEX_VBR_TAB :

                    chan.codec.nCodec = Codec.SPEEX_VBR_CODEC;
                    if (speexvbrnbRadioButton.Checked)
                        chan.codec.speex_vbr.nBandmode = 0;
                    else if (speexvbrwbRadioButton.Checked)
                        chan.codec.speex_vbr.nBandmode = 1;
                    else if (speexvbruwbRadioButton.Checked)
                        chan.codec.speex_vbr.nBandmode = 2;
                    chan.codec.speex_vbr.nQualityVBR = speexqualityTrackBar.Value;
                    chan.codec.speex_vbr.nMSecPerPacket = (int)speexvbrTxInterval.Value;
                    chan.codec.speex_vbr.bUseJitterBuffer = false;
                    chan.codec.speex_vbr.bStereoPlayback = false;
                    chan.codec.speex_vbr.nMaxBitRate = (int)speexvbrmaxbrNumericUpDown.Value;
                    chan.codec.speex_vbr.bDTX = speexvbrdtxCheckBox.Checked;
                    break;

                case (int)CodecTabs.CELT_CBR_TAB:

                    chan.codec.nCodec = Codec.CELT_CODEC;
                    if (celtkhz32RadioButton.Checked)
                        chan.codec.celt.nSampleRate = 32000;
                    else if (celtkhz441RadioButton.Checked)
                        chan.codec.celt.nSampleRate = 44100;
                    else if (celtkhz64RadioButton.Checked)
                        chan.codec.celt.nSampleRate = 64000;
                    chan.codec.celt.nBitRate = (int)celtbitrateNumericUpDown.Value * 1024;
                    chan.codec.celt.nChannels = celtchannelsComboBox.SelectedIndex + 1;
                    chan.codec.celt.nMSecPerPacket = (int)celtTxInterval.Value;
                    break;
                case (int)CodecTabs.CELT_VBR_TAB :

                    chan.codec.nCodec = Codec.CELT_VBR_CODEC;
                    if (celtvbrkhz32RadioButton.Checked)
                        chan.codec.celt_vbr.nSampleRate = 32000;
                    else if (celtvbrkhz441RadioButton.Checked)
                        chan.codec.celt_vbr.nSampleRate = 44100;
                    else if (celtvbrkhz64RadioButton.Checked)
                        chan.codec.celt_vbr.nSampleRate = 64000;
                    chan.codec.celt_vbr.nBitRate = (int)celtvbrbitrateNumericUpDown.Value * 1024;
                    chan.codec.celt_vbr.nChannels = celtvbrchannelsComboBox.SelectedIndex + 1;
                    chan.codec.celt_vbr.nMSecPerPacket = (int)celtvbrTxIntervalNumericUpDown.Value;
                    break;
            }
            chan.uChannelType |= staticCheckBox.Checked ? ChannelType.CHANNEL_STATIC : ChannelType.CHANNEL_DEFAULT;
            chan.uChannelType |= echoCheckBox.Checked ? ChannelType.CHANNEL_ECHO : ChannelType.CHANNEL_DEFAULT;
            chan.uChannelType |= singletxCheckBox.Checked ? ChannelType.CHANNEL_SOLO_TRANSMIT : ChannelType.CHANNEL_DEFAULT;
            chan.uChannelType |= classroomCheckBox.Checked ? ChannelType.CHANNEL_CLASSROOM: ChannelType.CHANNEL_DEFAULT;
            chan.uChannelType |= echovideoCheckBox.Checked ? ChannelType.CHANNEL_ECHO_VIDEO : ChannelType.CHANNEL_DEFAULT;
            chan.uChannelType |= echodesktopCheckBox.Checked ? ChannelType.CHANNEL_ECHO_DESKTOP : ChannelType.CHANNEL_DEFAULT;
            chan.uChannelType |= oprecvonlyCheckBox.Checked ? ChannelType.CHANNEL_OPERATOR_RECVONLY : ChannelType.CHANNEL_DEFAULT;

            chan.audiocfg.bEnableAGC = agcCheckBox.Checked;
            chan.audiocfg.nGainLevel = gainlevelTrackBar.Value;
            chan.audiocfg.nMaxIncDBSec = TeamTalk4.DEFAULT_AGC_INC_MAXDB;
            chan.audiocfg.nMaxDecDBSec = TeamTalk4.DEFAULT_AGC_DEC_MAXDB;
            chan.audiocfg.nMaxGainDB = TeamTalk4.DEFAULT_AGC_GAINMAXDB;
            chan.audiocfg.bEnableDenoise = denoiseCheckBox.Checked;
            chan.audiocfg.nMaxNoiseSuppressDB = TeamTalk4.DEFAULT_DENOISE_SUPPRESS;

            switch (dlgtype)
            {
                case ChannelDlgType.JOIN_CHANNEL :
                    if(ttclient.DoJoinChannel(chan)<0)
                        MessageBox.Show("Unable to join channel", "Join Channel");
                    //else wait for OnCmdChannelNew event
                    break;
                case ChannelDlgType.CREATE_CHANNEL :
                    if(ttclient.DoMakeChannel(chan)<0)
                        MessageBox.Show("Unable to create channel", "Create Channel");
                    //else wait for OnCmdChannelNew event
                    break;
                case ChannelDlgType.UPDATE_CHANNEL :
                    if(staticCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_STATIC;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_STATIC;
                    if (echoCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_ECHO;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_ECHO;
                    if (singletxCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_SOLO_TRANSMIT;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_SOLO_TRANSMIT;
                    if (classroomCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_CLASSROOM;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_CLASSROOM;
                    if (echovideoCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_ECHO_VIDEO;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_ECHO_VIDEO;
                    if (echodesktopCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_ECHO_DESKTOP;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_ECHO_DESKTOP;
                    if(oprecvonlyCheckBox.Checked)
                        chan.uChannelType |= ChannelType.CHANNEL_OPERATOR_RECVONLY;
                    else
                        chan.uChannelType &= ~ChannelType.CHANNEL_OPERATOR_RECVONLY;

                    if (ttclient.DoUpdateChannel(chan) < 0)
                        MessageBox.Show("Unable to update channel", "Update Channel");
                    break;
            }
        }

        private void channameTextBox_TextChanged(object sender, EventArgs e)
        {
            string path = "";
            ttclient.GetChannelPath(chan.nParentID, out path);
            path += channameTextBox.Text;
            chanpathTextBox.Text = path;
        }

        private void gainlevelTrackBar_ValueChanged(object sender, EventArgs e)
        {
            label11.Text = gainlevelTrackBar.Value.ToString();
        }
    }
}

